@@ -741,7 +741,7 @@ get_average_rate (GList  **rates,
 	if (g_list_length (*rates) > max_num) {
 		*rates = g_list_delete_link (*rates, *rates);
 	}
-	int_rate = (gint32)round (scale * rate);
+	int_rate = (gint32)ceil (scale * rate);
 
 	*rates = g_list_append (*rates, GINT_TO_POINTER (int_rate));
 
