@@ -1284,7 +1284,93 @@ nautilus_burn_drive_eject (NautilusBurnDrive *drive)
 
 #ifdef USE_HAL
 
-#define GET_BOOL_PROP(x) (libhal_device_property_exists (ctx, device_names [i], x, NULL) && libhal_device_get_property_bool (ctx, device_names [i], x, NULL))
+#define LIBHAL_PROP_EXTRACT_BEGIN if (FALSE)
+#define LIBHAL_PROP_EXTRACT_END ;
+#define LIBHAL_PROP_EXTRACT_INT(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_INT32) _where_ = libhal_psi_get_int (&it)
+#define LIBHAL_PROP_EXTRACT_INT_DIV(_property_, _where_, _div_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_INT32) _where_ = libhal_psi_get_int (&it) / _div_
+#define LIBHAL_PROP_EXTRACT_STRING(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_STRING) _where_ = (libhal_psi_get_string (&it) != NULL && strlen (libhal_psi_get_string (&it)) > 0) ? strdup (libhal_psi_get_string (&it)) : NULL
+#define LIBHAL_PROP_EXTRACT_BOOL(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_BOOLEAN) _where_ = libhal_psi_get_bool (&it)
+#define LIBHAL_PROP_EXTRACT_BOOL_BITFIELD(_property_, _where_, _field_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_BOOLEAN) _where_ |= libhal_psi_get_bool (&it) ? _field_ : 0
+#define LIBHAL_PROP_EXTRACT_STRLIST(_property_, _where_) else if (strcmp (key, _property_) == 0 && type == LIBHAL_PROPERTY_TYPE_STRLIST) _where_ = my_strvdup (libhal_psi_get_strlist (&it))
+
+/** Checks if LibHalContext *ctx == NULL */
+#ifndef LIBHAL_CHECK_LIBHALCONTEXT
+#define LIBHAL_CHECK_LIBHALCONTEXT(_ctx_, _ret_)					\
+	do {									\
+		if (_ctx_ == NULL) {						\
+			fprintf (stderr,					\
+				 "%s %d : LibHalContext *ctx is NULL\n", 	\
+				 __FILE__, __LINE__);				\
+			return _ret_;						\
+		}								\
+	} while(0)
+#endif
+
+static NautilusBurnDrive *
+hal_drive_from_udi (LibHalContext *ctx,
+		    const char    *udi)
+{
+	LibHalPropertySet        *pset;
+	LibHalPropertySetIterator it;
+	DBusError                 error;
+	NautilusBurnDrive        *drive;
+
+	LIBHAL_CHECK_LIBHALCONTEXT (ctx, FALSE);
+
+	dbus_error_init (&error);
+
+	if ((pset = libhal_device_get_all_properties (ctx, udi, &error)) == NULL) {
+		if (dbus_error_is_set (&error)) {
+			g_warning ("Could not get all properties: %s", error.message);
+			dbus_error_free (&error);
+		}
+
+		return NULL;
+	}
+
+	drive = nautilus_burn_drive_new ();
+	drive->type = NAUTILUS_BURN_DRIVE_TYPE_CD_DRIVE;
+
+	for (libhal_psi_init (&it, pset); libhal_psi_has_more (&it); libhal_psi_next (&it)) {
+		int   type;
+		char *key;
+
+		type = libhal_psi_get_type (&it);
+		key = libhal_psi_get_key (&it);
+
+		LIBHAL_PROP_EXTRACT_BEGIN;
+
+		LIBHAL_PROP_EXTRACT_STRING  ("block.device",              drive->device);
+		LIBHAL_PROP_EXTRACT_STRING  ("storage.model",             drive->display_name);
+
+		LIBHAL_PROP_EXTRACT_INT_DIV ("storage.cdrom.read_speed",  drive->max_speed_read,  CD_ROM_SPEED);
+		LIBHAL_PROP_EXTRACT_INT_DIV ("storage.cdrom.write_speed", drive->max_speed_write, CD_ROM_SPEED);
+
+
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.cdr",        drive->type, NAUTILUS_BURN_DRIVE_TYPE_CD_RECORDER);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.cdrw",       drive->type, NAUTILUS_BURN_DRIVE_TYPE_CDRW_RECORDER);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvd",        drive->type, NAUTILUS_BURN_DRIVE_TYPE_DVD_DRIVE);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdplusr",   drive->type, NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_RECORDER);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdplusrw",  drive->type, NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_RW_RECORDER);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdplusrdl", drive->type, NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_DL_RECORDER);
+
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdr",       drive->type, NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdrw",      drive->type, NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdram",     drive->type, NAUTILUS_BURN_DRIVE_TYPE_DVD_RAM_RECORDER);
+
+		LIBHAL_PROP_EXTRACT_END;
+	}
+
+	libhal_free_property_set (pset);
+
+	drive->priv->udi = g_strdup (udi);
+	drive->cdrecord_id = g_strdup (drive->device);
+
+	if (! drive->display_name)
+		drive->display_name = g_strdup_printf ("Unnamed Drive (%s)", drive->device);
+
+	return drive;
+}
 
 static GList *
 hal_scan (gboolean recorder_only)
@@ -1310,78 +1396,8 @@ hal_scan (gboolean recorder_only)
 
 	for (i = 0; i < num_devices; i++) {
 		NautilusBurnDrive *drive;
-		char              *string;
-		gboolean           is_cdr;
 
-		/* Is it a CD burner? */
-		is_cdr = GET_BOOL_PROP ("storage.cdrom.cdr");
-
-		drive = nautilus_burn_drive_new ();
-		drive->type = NAUTILUS_BURN_DRIVE_TYPE_CD_DRIVE;
-
-		if (is_cdr != FALSE) {
-			drive->type |= NAUTILUS_BURN_DRIVE_TYPE_CD_RECORDER;
-		}
-
-		if (GET_BOOL_PROP ("storage.cdrom.cdrw")) {
-			drive->type |= NAUTILUS_BURN_DRIVE_TYPE_CDRW_RECORDER;
-		}
-
-		if (GET_BOOL_PROP ("storage.cdrom.dvd")) {
-			drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_DRIVE;
-
-			if (GET_BOOL_PROP ("storage.cdrom.dvdram")) {
-				drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_RAM_RECORDER;
-			}
-
-			if (GET_BOOL_PROP ("storage.cdrom.dvdr")) {
-				drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER;
-			}
-
-			if (GET_BOOL_PROP ("storage.cdrom.dvd")) {
-				drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_DRIVE;
-			}
-
-			if (GET_BOOL_PROP ("storage.cdrom.dvdplusr")) {
-				drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_RECORDER;
-			}
-
-			if (GET_BOOL_PROP ("storage.cdrom.dvdplusrw")) {
-				drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_RW_RECORDER;
-			}
-
-		}
-
-		drive->device = libhal_device_get_property_string (ctx,
-								   device_names [i], 
-								   "block.device",
-								   NULL);
-
-		drive->cdrecord_id = g_strdup (drive->device);
-
-		string = libhal_device_get_property_string (ctx,
-							    device_names [i], 
-							    "storage.model",
-							    NULL);
-
-		if (string != NULL) {
-			drive->display_name = string;
-		} else {
-			drive->display_name = g_strdup_printf ("Unnamed Drive (%s)", drive->device);
-		}
-
-		drive->max_speed_read = libhal_device_get_property_int
-			(ctx, device_names [i], "storage.cdrom.read_speed", NULL)
-			/ CD_ROM_SPEED;
-
-		if (libhal_device_property_exists (ctx, device_names [i], "storage.cdrom.write_speed", NULL)) {
-			drive->max_speed_write = libhal_device_get_property_int
-				(ctx, device_names [i],
-				 "storage.cdrom.write_speed",
-				 NULL)
-				/ CD_ROM_SPEED;
-
-		}
+		drive = hal_drive_from_udi (ctx, device_names [i]);
 
 		add_whitelist (drive);
 
@@ -1390,8 +1406,6 @@ hal_scan (gboolean recorder_only)
 		} else {
 			drives = g_list_prepend (drives, drive);
 		}
-
-		drive->priv->udi = g_strdup (device_names [i]);
 	}
 
 	libhal_free_string_array (device_names);
