@@ -69,7 +69,7 @@ struct _NautilusBurnProcess {
 	GString                  *line_stderr;
 	NautilusBurnProcessFuncs *funcs;
 
-	int                       stdin;
+	int                       pipe_stdin;
 
 	time_t                    start_time;
 	gint64                    start_num;
@@ -235,6 +235,7 @@ can_burn_dvds (NautilusBurnDrive *recorder)
 {
 	if (!(recorder->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER) &&
 	    !(recorder->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_RECORDER) &&
+	    !(recorder->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_DL_RECORDER) &&
 	    !(recorder->type & NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_RW_RECORDER)) {
 		return FALSE;
 	}
@@ -309,7 +310,7 @@ insert_cd_retry (NautilusBurnProcess *process,
 		nautilus_burn_process_cancel (process, FALSE);
 	} else if (is_reload) {
 		if (send_return) {
-			write (process->stdin, "\n", 1);
+			write (process->pipe_stdin, "\n", 1);
 		} else {
 			kill (process->pid, SIGUSR1);
 		}
@@ -326,7 +327,7 @@ compute_time_remaining (gint64 bytes,
 {
 	long secs;
 
-	if (bytes_per_sec <= 0)
+	if (bytes_per_sec <= 1)
 		return -1;
 
 	secs = bytes / bytes_per_sec;
@@ -1248,7 +1249,7 @@ nautilus_burn_recorder_run_process (NautilusBurnRecorder       *recorder,
 				      recorder,
 				      &stdout_tag,
 				      &stderr_tag,
-				      &process->stdin,
+				      &process->pipe_stdin,
 				      &local_error);
 
 	if (! res) {
@@ -1463,26 +1464,28 @@ nautilus_burn_recorder_write_cdrecord (NautilusBurnRecorder          *recorder,
 
 	track = tracks->data;
 
+	if (flags & NAUTILUS_BURN_RECORDER_WRITE_BLANK) {
+		NautilusBurnRecorderBlankFlags blank_flags = 0;
+		int                            res;
+		GError                        *err = NULL;
+
+		if (flags & NAUTILUS_BURN_RECORDER_WRITE_DEBUG)
+			blank_flags |= NAUTILUS_BURN_RECORDER_BLANK_DEBUG;
+		if (flags & NAUTILUS_BURN_RECORDER_WRITE_DUMMY_WRITE)
+			blank_flags |= NAUTILUS_BURN_RECORDER_BLANK_DUMMY_WRITE;
+		/* cdrdao doesn't seem to have on-the-fly blank */
+		/* cdrecord doesn't seem to correctly detect disc size when blanking */
+		res = nautilus_burn_recorder_blank_disc (recorder,
+							 drive,
+							 NAUTILUS_BURN_RECORDER_BLANK_FAST,
+							 blank_flags,
+							 &err);
+		if (res != NAUTILUS_BURN_RECORDER_RESULT_FINISHED)
+			return res;
+	}
+
+
 	if (track->type ==  NAUTILUS_BURN_RECORDER_TRACK_TYPE_CUE) {
-		if (flags & NAUTILUS_BURN_RECORDER_WRITE_BLANK) {
-			NautilusBurnRecorderBlankFlags blank_flags = 0;
-			int                            res;
-			GError                        *err = NULL;
-
-			if (flags & NAUTILUS_BURN_RECORDER_WRITE_DEBUG)
-				blank_flags |= NAUTILUS_BURN_RECORDER_BLANK_DEBUG;
-			if (flags & NAUTILUS_BURN_RECORDER_WRITE_DUMMY_WRITE)
-				blank_flags |= NAUTILUS_BURN_RECORDER_BLANK_DUMMY_WRITE;
-			/* cdrdao doesn't seem to have on-the-fly blank */
-			res = nautilus_burn_recorder_blank_disc (recorder,
-								 drive,
-								 NAUTILUS_BURN_RECORDER_BLANK_FAST,
-								 blank_flags,
-								 &err);
-			if (res != NAUTILUS_BURN_RECORDER_RESULT_FINISHED)
-				return res;
-		}
-
 		argv = g_ptr_array_new ();
 		g_ptr_array_add (argv, "cdrdao");
 		g_ptr_array_add (argv, "write");
@@ -1527,9 +1530,6 @@ nautilus_burn_recorder_write_cdrecord (NautilusBurnRecorder          *recorder,
 		if (flags & NAUTILUS_BURN_RECORDER_WRITE_EJECT) {
 			g_ptr_array_add (argv, "-eject");
 		}
-		if (flags & NAUTILUS_BURN_RECORDER_WRITE_BLANK) {
-			g_ptr_array_add (argv, "blank=fast");
-		}
 		if (flags & NAUTILUS_BURN_RECORDER_WRITE_DISC_AT_ONCE) {
 			g_ptr_array_add (argv, "-dao");
 		}
